@@ -1,46 +1,53 @@
 'use strict';
 
-var assert = require('assert');
-var fs = require('fs');
-var path = require('path');
-var globby = require('globby');
-var multimatch = require('multimatch');
-var ejs = require('ejs');
-var util = require('../util');
-var normalize = require('normalize-path');
+const assert = require('assert');
+const fs = require('fs');
+const path = require('path');
+const globby = require('globby');
+const multimatch = require('multimatch');
+const ejs = require('ejs');
+const util = require('../util');
+const normalize = require('normalize-path');
 
 function applyProcessingFunc(process, contents, filename) {
-  var output = process(contents, filename);
+  const output = process(contents, filename);
   return Buffer.isBuffer(output) ? output : Buffer.from(output);
 }
 
 exports.copy = function (from, to, options, context, tplSettings) {
   to = path.resolve(to);
   options = options || {};
-  var fromGlob = util.globify(from);
+  const fromGlob = util.globify(from);
 
-  var globOptions = {...options.globOptions, nodir: true};
-  var diskFiles = globby.sync(fromGlob, globOptions).map(file => path.resolve(file));
-  var storeFiles = [];
+  let diskFiles = [];
+  if (options.noGlob) {
+    const fromFiles = Array.isArray(fromGlob) ? fromGlob : [fromGlob];
+    diskFiles = fromFiles.filter(filepath => fs.existsSync(filepath));
+  } else {
+    const globOptions = {...options.globOptions, nodir: true};
+    diskFiles = globby.sync(fromGlob, globOptions).map(file => path.resolve(file));
+  }
+
+  const storeFiles = [];
   this.store.each(file => {
     // The store may have a glob path and when we try to copy it will fail because not real file
     if (!globby.hasMagic(normalize(file.path)) && multimatch([file.path], fromGlob).length !== 0 && !diskFiles.includes(file.path)) {
       storeFiles.push(file.path);
     }
   });
-  var files = diskFiles.concat(storeFiles);
+  const files = diskFiles.concat(storeFiles);
 
-  var generateDestination = () => to;
+  let generateDestination = () => to;
   if (Array.isArray(from) || !this.exists(from) || globby.hasMagic(normalize(from))) {
     assert(
       !this.exists(to) || fs.statSync(to).isDirectory(),
-      'When copying multiple files, provide a directory as destination'
+      'When copying multiple files, provide a directory as destination',
     );
 
     const processDestinationPath = options.processDestinationPath || (path => path);
-    var root = util.getCommonPath(from);
+    const root = util.getCommonPath(from);
     generateDestination = filepath => {
-      var toFile = path.relative(root, filepath);
+      const toFile = path.relative(root, filepath);
       return processDestinationPath(path.join(to, toFile));
     };
   }
@@ -49,7 +56,7 @@ exports.copy = function (from, to, options, context, tplSettings) {
   assert(options.ignoreNoMatch || files.length > 0, 'Trying to copy from a source that does not exist: ' + from);
 
   files.forEach(file => {
-    var toFile = generateDestination(file);
+    let toFile = generateDestination(file);
     if (context) {
       toFile = ejs.render(toFile, context, tplSettings);
     }
@@ -61,9 +68,9 @@ exports.copy = function (from, to, options, context, tplSettings) {
 exports._copySingle = function (from, to, options = {}) {
   assert(this.exists(from), 'Trying to copy from a source that does not exist: ' + from);
 
-  var file = this.store.get(from);
+  const file = this.store.get(from);
 
-  var contents = file.contents;
+  let {contents} = file;
   if (options.process) {
     contents = applyProcessingFunc(options.process, file.contents, file.path);
   }
