@@ -1,7 +1,7 @@
-'use strict';
 const fs = require('fs');
 const path = require('path');
 const EventEmitter = require('events');
+const crypto = require('crypto');
 const chalk = require('chalk');
 const _ = require('lodash');
 const GroupedQueue = require('grouped-queue');
@@ -11,13 +11,11 @@ const memFs = require('mem-fs');
 const FileEditor = require('mem-fs-editor');
 const debug = require('debug')('yeoman:environment');
 const isScoped = require('is-scoped');
-const crypto = require('crypto');
 const npmlog = require('npmlog');
+const semver = require('semver');
+const slash = require('slash');
 const {TrackerGroup} = require('are-we-there-yet');
-
-const {promisify} = require('util');
-const {pipeline: _pipeline} = require('stream');
-const pipeline = promisify(_pipeline);
+const {pipeline, transform} = require('p-transform');
 
 const ENVIRONMENT_VERSION = require('../package.json').version;
 const Store = require('./store');
@@ -31,7 +29,6 @@ const {
   createCommitTransform,
   createConflicterCheckTransform,
   createConflicterStatusTransform,
-  createEachFileTransform,
   createModifiedTransform,
   createYoRcTransform,
   createYoResolveTransform
@@ -47,13 +44,13 @@ function splitArgsFromString(argsString) {
     return result;
   }
   const quoteSeparatedArgs = argsString.split(/("[^"]*")/).filter(x => x);
-  quoteSeparatedArgs.forEach(arg => {
+  for (const arg of quoteSeparatedArgs) {
     if (arg.match('\x22')) {
       result.push(arg.replace(/"/g, ''));
     } else {
       result = result.concat(arg.trim().split(' '));
     }
-  });
+  }
   return result;
 }
 
@@ -164,7 +161,7 @@ class Environment extends Base {
       rootCommand.emit('yeoman:environment', command.env);
 
       const generator = command.env.instantiate(GeneratorClass, this.args, this.opts());
-      command.env.queueGenerator(generator);
+      await command.env.queueGenerator(generator);
       return command.env.start().then(() => command.env);
     });
     return command;
@@ -332,12 +329,12 @@ class Environment extends Base {
     this.repository = new YeomanRepository(this.options.yeomanRepository);
 
     if (!this.options.experimental) {
-      process.argv.forEach(value => {
+      for (const value of process.argv) {
         if (value === '--experimental') {
           this.options.experimental = true;
           debug('Set environment as experimental');
         }
-      });
+      }
     }
 
     this.loadSharedOptions(this.options);
@@ -351,6 +348,8 @@ class Environment extends Base {
 
     // Store the YeomanCompose by paths and uniqueBy feature.
     this._composeStore = {};
+
+    this.enableConflicterIgnore = semver.satisfies(this.getVersion('mem-fs-editor'), '>= 9.2.0');
   }
 
   /**
@@ -583,6 +582,9 @@ class Environment extends Base {
   getPackagePath(namespace) {
     if (namespace.includes(':')) {
       const generator = this.get(namespace) || {};
+      if (generator.then) {
+        return generator.then(generator => generator.packagePath);
+      }
       return generator.packagePath;
     }
     const packagePaths = this.getPackagePaths(namespace) || [];
@@ -750,7 +752,7 @@ class Environment extends Base {
           'You can see available generators via ' +
           chalk.yellow('npm search yeoman-generator') + ' or via ' + chalk.yellow('http://yeoman.io/generators/') + '. \n' +
           'Install them with ' + chalk.yellow(`npm install ${generatorHint}`) + '.\n\n' +
-          'To see all your installed generators run ' + chalk.yellow('yo') + ' without any arguments. ' +
+          'To see all your installed generators run ' + chalk.yellow('yo --generators') + '. ' +
           'Adding the ' + chalk.yellow('--help') + ' option will also show subgenerators. \n\n' +
           'If ' + chalk.yellow('yo') + ' cannot find the generator, run ' + chalk.yellow('yo doctor') + ' to troubleshoot your system.'
         );
@@ -759,8 +761,16 @@ class Environment extends Base {
     };
 
     maybeGenerator = maybeGenerator || this.get(namespaceOrPath);
-    if (maybeGenerator && maybeGenerator.then) {
-      return maybeGenerator.then(Generator => checkGenerator(Generator)).then(Generator => this.instantiate(Generator, args, options));
+    if (maybeGenerator && (maybeGenerator.then || maybeGenerator.prototype._postConstruct)) {
+      return Promise.resolve(maybeGenerator)
+        .then(Generator => checkGenerator(Generator))
+        .then(Generator => this.instantiate(Generator, args, options))
+        .then(async generator => {
+          if (!options.help && generator._postConstruct) {
+            await generator._postConstruct();
+          }
+          return generator;
+        });
     }
 
     return this.instantiate(checkGenerator(maybeGenerator), args, options);
@@ -812,14 +822,23 @@ class Environment extends Base {
    * @param {String} namespaceOrPath
    * @param {Array} [args]
    * @param {Object} [options]
+   * @param {Boolean} [schedule]
    * @return {Generator} The instantiated generator or the singleton instance.
    */
-  composeWith(generator, args, options) {
+  composeWith(generator, args, options, schedule = true) {
+    if (typeof args === 'boolean') {
+      schedule = args;
+      args = undefined;
+      options = undefined;
+    } else if (typeof options === 'boolean') {
+      schedule = options;
+      options = undefined;
+    }
     const generatorInstance = this.create(generator, args, options);
     if (generatorInstance.then) {
-      return generatorInstance.then(generatorInstance => this.queueGenerator(generatorInstance, true));
+      return generatorInstance.then(generatorInstance => this.queueGenerator(generatorInstance, schedule));
     }
-    return this.queueGenerator(generatorInstance, true);
+    return this.queueGenerator(generatorInstance, schedule);
   }
 
   /**
@@ -846,9 +865,7 @@ class Environment extends Base {
   getAllGenerators() {
     return Object.fromEntries([
       ...Object.entries(this._generators),
-      ...Object.entries(this._generatorsForPath).map(([root, generatorStore]) => {
-        return Object.entries(generatorStore).map(([namespace, generator]) => ([`${root}#${namespace}`, generator]));
-      }).flat()
+      ...Object.entries(this._generatorsForPath).flatMap(([root, generatorStore]) => Object.entries(generatorStore).map(([namespace, generator]) => ([`${root}#${namespace}`, generator])))
     ]);
   }
 
@@ -913,21 +930,20 @@ class Environment extends Base {
         // Generator > 5
         this.once('run', () => generator.emit('run'));
         this.once('end', () => generator.emit('end'));
-        generator.queueTasks();
-      } else {
-        if (!generator.options.forwardErrorToEnvironment) {
-          generator.on('error', error => this.emit('error', error));
-        }
-        generator.promise = generator.run();
+        return generator.queueTasks();
       }
+      if (!generator.options.forwardErrorToEnvironment) {
+        generator.on('error', error => this.emit('error', error));
+      }
+      generator.promise = generator.run();
     };
 
     if (schedule) {
       this.runLoop.add(
         'environment:run',
-        (done, stop) => {
+        async (done, stop) => {
           try {
-            runGenerator();
+            await runGenerator();
             done();
           } catch (error) {
             stop(error);
@@ -935,7 +951,10 @@ class Environment extends Base {
         }
       );
     } else {
-      runGenerator();
+      const maybePromise = runGenerator();
+      if (maybePromise && maybePromise.then) {
+        return maybePromise.then(() => generator);
+      }
     }
     return generator;
   }
@@ -950,7 +969,7 @@ class Environment extends Base {
    * @param {Object}       [options]
    */
   async run(args, options, done) {
-    if (done === 'function' || typeof options === 'function' || typeof args === 'function') {
+    if (done || typeof options === 'function' || typeof args === 'function') {
       throw new Error('Callback support have been removed.');
     }
 
@@ -1056,7 +1075,7 @@ class Environment extends Base {
   async runGenerator(generator) {
     try {
       generator = await generator;
-      generator = this.queueGenerator(generator);
+      generator = await this.queueGenerator(generator);
     } catch (error) {
       return Promise.reject(error);
     }
@@ -1101,28 +1120,39 @@ class Environment extends Base {
       throw new Error('Missing namespace');
     }
 
+    // Normalize path
+    let ns = slash(filepath);
+
+    // Ignore path before latest node_modules
+    const REPOSITORY_PATH = '/node_modules/';
+    if (ns.includes(REPOSITORY_PATH)) {
+      ns = ns.slice(ns.lastIndexOf(REPOSITORY_PATH) + REPOSITORY_PATH.length, ns.length);
+    }
+
     // Cleanup extension and normalize path for differents OS
-    let ns = path.normalize(filepath.replace(new RegExp(escapeStrRe(path.extname(filepath)) + '$'), ''));
+    const parsed = path.parse(ns);
+    ns = parsed.dir ? `${parsed.dir}/${parsed.name}` : parsed.name;
 
     // Sort lookups by length so biggest are removed first
-    const nsLookups = _(lookups.concat(['..'])).map(found => path.normalize(found)).sortBy('length').value().reverse();
+    const nsLookups = _([...lookups, '..']).map(found => slash(found)).sortBy('length').value().reverse();
 
     // If `ns` contains a lookup dir in its path, remove it.
-    ns = nsLookups.reduce((ns, lookup) => {
+    for (let lookup of nsLookups) {
       // Only match full directory (begin with leading slash or start of input, end with trailing slash)
-      lookup = new RegExp(`(?:\\\\|/|^)${escapeStrRe(lookup)}(?=\\\\|/)`, 'g');
-      return ns.replace(lookup, '');
-    }, ns);
+      lookup = new RegExp(`(?:/|^)${escapeStrRe(lookup)}(?=/)`, 'g');
+      ns = ns.replace(lookup, '');
+    }
 
-    const folders = ns.split(path.sep);
+    const folders = ns.split('/');
     const scope = _.findLast(folders, folder => folder.indexOf('@') === 0);
 
     // Cleanup `ns` from unwanted parts and then normalize slashes to `:`
     ns = ns
+      .replace(/\/\//g, '') // Remove double `/`
       .replace(/(.*generator-)/, '') // Remove before `generator-`
-      .replace(/[/\\](index|main)$/, '') // Remove `/index` or `/main`
-      .replace(/^[/\\]+/, '') // Remove leading `/`
-      .replace(/[/\\]+/g, ':'); // Replace slashes by `:`
+      .replace(/\/(index|main)$/, '') // Remove `/index` or `/main`
+      .replace(/^\//, '') // Remove leading `/`
+      .replace(/\/+/g, ':'); // Replace slashes by `:`
 
     if (scope) {
       ns = `${scope}/${ns}`;
@@ -1188,12 +1218,12 @@ class Environment extends Base {
       stream,
       createModifiedTransform(),
       ...transformStreams,
-      createEachFileTransform(file => {
+      transform(file => {
         if (log) {
           log.completeWork(10);
           npmlog.info('Completed', path.relative(this.logCwd, file.path));
         }
-      }, {autoForward: false, logName: 'environment:log'})
+      }, 'environment:log')
     ).then(() => {
       if (log) {
         log.finish();
@@ -1210,10 +1240,23 @@ class Environment extends Base {
   commitSharedFs(stream = this.sharedFs.stream()) {
     return new Promise((resolve, reject) => {
       debug('committing files');
+
+      const conflicterStatus = {};
+      if (this.enableConflicterIgnore) {
+        conflicterStatus.fileActions = [{
+          key: 'i',
+          name: 'ignore, do not overwrite and remember (experimental)',
+          value: ({relativeFilePath}) => {
+            this.fs.append(`${this.cwd}/.yo-resolve`, `${relativeFilePath} skip`, {create: true});
+            return 'skip';
+          }
+        }];
+      }
+
       this.fs.commit([
         createYoResolveTransform(this.conflicter),
         createYoRcTransform(),
-        createConflicterCheckTransform(this.conflicter),
+        createConflicterCheckTransform(this.conflicter, conflicterStatus),
         createConflicterStatusTransform(),
         // Use custom commit transform due to out of order transform.
         createCommitTransform(this.fs)
@@ -1225,10 +1268,7 @@ class Environment extends Base {
           reject(error);
           return;
         }
-
-        // Force to empty Conflicter queue.
-        this.conflicter.queue.once('end', () => resolve(value));
-        this.conflicter.queue.run();
+        resolve(value);
       });
     });
   }
@@ -1249,14 +1289,21 @@ class Environment extends Base {
         } else {
           customCommitTask = this.commitSharedFs.bind(this);
         }
+
+        if (this.enableConflicterIgnore) {
+          debug('Adding queueCommit event listener');
+          this.sharedFs.once('change', queueCommit);
+        }
         const result = customCommitTask();
         if (!result || !result.then) {
           done();
           return;
         }
         return result.then(() => {
-          debug('Adding queueCommit event listener');
-          this.sharedFs.once('change', queueCommit);
+          if (!this.enableConflicterIgnore) {
+            debug('Adding queueCommit event listener');
+            this.sharedFs.once('change', queueCommit);
+          }
           done();
         }
         , stop);
