@@ -9,13 +9,12 @@
  ***********************************************************************/
 
 import * as fs from 'fs-extra';
-import * as mustache from 'mustache';
 import * as path from 'path';
 import * as readPkg from 'read-pkg';
 
 import { Command } from './command';
 import { ISource } from './init-sources';
-import { Logger } from './logger';
+import { generateAssembly } from './generate-assembly';
 
 /**
  * Generates the examples/assembly
@@ -36,44 +35,19 @@ export class Init {
         return (await readPkg(path.join(this.rootFolder, 'packages/core/package.json'))).version;
     }
 
-    async getPackageWithVersion(name: string): Promise<string> {
-        const pkg = JSON.parse(
-            await new Command(path.resolve(this.rootFolder)).exec(Init.GET_PACKAGE_WITH_VERSION_CMD + name)
-        ).data.trees[0];
-        return pkg ? pkg.name : '';
-    }
-
-    async generate(): Promise<void> {
-        const srcDir = path.resolve(__dirname, '../src');
-        const distDir = path.resolve(__dirname, '../dist');
-        const templateDir = path.join(srcDir, 'templates');
-        const compileTsConfig = path.join(templateDir, 'assembly-compile.tsconfig.json');
-        const packageJsonContent = await fs.readFile(path.join(templateDir, 'assembly-package.mst'));
-
-        // generate assembly if does not exists
-        const rendered = await this.generateAssemblyPackage(packageJsonContent.toString());
-        await fs.ensureDir(this.examplesAssemblyFolder);
-        await fs.writeFile(path.join(this.examplesAssemblyFolder, 'package.json'), rendered);
-        await fs.copy(compileTsConfig, path.join(this.examplesAssemblyFolder, 'compile.tsconfig.json'));
-        await fs.copy(path.join(templateDir, 'cdn'), path.join(this.examplesAssemblyFolder, 'cdn'));
-        Logger.info(distDir);
-        await fs.copy(path.join(distDir, 'cdn'), path.join(this.examplesAssemblyFolder, 'cdn'));
-        await fs.copy(path.join(srcDir, 'scripts'), path.join(this.examplesAssemblyFolder, 'scripts'));
-
+    async generate() {
+        await generateAssembly(this.examplesAssemblyFolder, {
+            theiaVersion: '^' + (await this.getCurrentVersion()),
+            monacoVersion: await this.getPackageWithVersion(Init.MONACO_CORE_PKG),
+            configDirPrefix: '../../packages/@che-',
+            packageRefPrefix: '../../config/',
+        });
         // Generate checkout folder is does not exist
         await fs.ensureDir(this.checkoutFolder);
 
         // copy build all plugins scripts
         await fs.ensureDir(this.pluginsFolder);
-        await fs.copy(path.join(srcDir, 'foreach_yarn'), path.join(this.pluginsFolder, 'foreach_yarn'));
-    }
-
-    async generateAssemblyPackage(template: string): Promise<string> {
-        const tags = {
-            version: await this.getCurrentVersion(),
-            monacopkg: await this.getPackageWithVersion(Init.MONACO_CORE_PKG),
-        };
-        return mustache.render(template, tags).replace(/&#x2F;/g, '/');
+        await fs.copy(path.resolve(__dirname, '../src/foreach_yarn'), path.join(this.pluginsFolder, 'foreach_yarn'));
     }
 
     async updadeBuildConfiguration(extensions: ISource[]): Promise<void> {
@@ -126,21 +100,28 @@ export class Init {
         await fs.writeFile(theiaPackagePath, json);
     }
 
-    async updatePluginsConfigurtion(): Promise<void> {
+    async updatePluginsConfiguration(): Promise<void> {
         const theiaPackagePath = path.join(this.rootFolder, 'package.json');
         const theiaPackage = await readPkg(theiaPackagePath);
 
-        theiaPackage['theiaPlugins'] = await this.getPluginsList();
+        theiaPackage['theiaPlugins'] = await Init.getPluginsList();
 
         const json = JSON.stringify(theiaPackage, undefined, 2);
         await fs.writeFile(theiaPackagePath, json);
     }
 
     // eslint-disable-next-line @typescript-eslint/no-explicit-any
-    private async getPluginsList(): Promise<any> {
+    private static async getPluginsList(): Promise<any> {
         const srcDir = path.resolve(__dirname, '../src');
         const templateDir = path.join(srcDir, 'templates');
         const pluginsJsonContent = await fs.readFile(path.join(templateDir, 'theiaPlugins.json'));
         return JSON.parse(pluginsJsonContent.toString());
     }
+
+    async getPackageWithVersion(name: string): Promise<string> {
+        const pkg = JSON.parse(
+            await new Command(path.resolve(this.rootFolder)).exec(Init.GET_PACKAGE_WITH_VERSION_CMD + name)
+        ).data.trees[0];
+        return pkg ? pkg.name : '';
+    }
 }
